package directory.passive.markov;

import java.util.AbstractMap;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ThreadLocalRandom;

public class Builder {
	
	private final int prefix_length;
	private final Data data;

	public Builder(int prefix_length, Data data) {
		super();
		this.prefix_length = prefix_length;
		this.data = data;
	}

	public Collection<Map.Entry<Prefix, Token>> random() {
		List<Map.Entry<Prefix, Token>> result = new LinkedList<>();
		Token[] initToken = new Token[prefix_length];
		for (int i = 0; i < prefix_length; i++) {
			initToken[i] = Token.START;
		}
		Prefix p = new Prefix(initToken);
		Token t = Token.START;
		while (t != Token.END) {
			t = nextRandom(p);
			result.add(new AbstractMap.SimpleEntry<Prefix, Token>(p, t));
			p = p.slide(t);
		}
		return result;
	}

	private Token nextRandom(Prefix prefix) {
		return data.fetch(prefix).forRandom(nextRandomNumber());
	}

	private double nextRandomNumber() {
		return ThreadLocalRandom.current().nextDouble();
	}
}
